@@ -1,16 +1,23 @@
 #define banner  \
-"bg5conv (CJK ver. 4.8.4)" \
+"bg5conv (CJK ver. 4.8.5)" \
 
 /*2:*/
-#line 94 "./cjkutils-4.8.4/Bg5conv/bg5conv.w"
+#line 94 "./cjkutils-src/Bg5conv/bg5conv.w"
 
 #include <stdio.h> 
 #include <stdlib.h> 
+#ifdef WIN32
+#include <fcntl.h> 
+#include <io.h> 
+#endif
 
 
 int main(int argc,char*argv[])
 {int ch;
 
+#ifdef WIN32
+setmode(fileno(stdout),_O_BINARY);
+#endif
 fprintf(stdout,"\\def\\CJKpreproc{%s}",banner);
 
 ch= fgetc(stdin);
