@@ -96,6 +96,8 @@
 #include <ptexenc/unicode.h>
 #endif
 
+#include "common.h"
+
 #define uchar unsigned char
 #define uint  unsigned int
 #define Long  int
@@ -312,8 +314,8 @@ const int AdID = (('A'<<24)+('d'<<16)+('O'<<8)+EOP);
 #ifndef PTEXENC
 // #define issjis1(c) ((c)>=0x81&&(c)<=0xfc&&((c)<=0x9f||(c)>=0xe0))
 // #define issjis2(c) ((c)>=0x40 && (c)<=0xfc && (c)!=0x7f)
-#define isjis(c) (((c)>=0x21 && (c)<=0x7e))
 #endif
+#define isjis(c) (((c)>=0x21 && (c)<=0x7e))
 #define is_hex(c)   ((c>='0'&&c<='9')||(c>='a'&&c<='f')||(c>='A'&&c<='F'))
 #define is_oct(c)   (c>='0'&&c<='7')
 // #define is_dig(c)   (c>='0'&&c<='9')
@@ -471,6 +473,7 @@ void usage(int ex)
     "   pdf   specials:  pdf:bcolor, pdf:ecolor, pdf:bgcolor\n"
     "   tpic  specials:  pn\n"
     );
+    fprintf(stderr, "\nEmail bug reports to %s.\n", BUG_ADDRESS);
     exit(ex);
 }
 
@@ -784,7 +787,11 @@ same:       strcpy(outfile, infile);
         else if(f_mode == EXE2MODIFY)
             fp_out = stderr;
         else{
+#ifndef UNIX
+            fp_out = fopen(outfile, WRITE_BINARY);
+#else
             fp_out = fopen(outfile, WRITE_TEXT);
+#endif
             if(fp_out == NULL){
                 fprintf(stderr, "Cannot open %s for output\n", outfile);
                 exit(1);
@@ -1909,6 +1916,8 @@ uint work(FILE *dvi)
                     code = read_n(dvi, mode & 0xf);
 #ifdef PTEXENC
                     if(f_jstr){
+                      // internal-euc/sjis: fromDVI cannot convert ASCII range
+                      if (is_internalUPTEX() || (isjis(code>>8) && isjis(code&0xff))) {
                         wch = fromDVI(code);
                         if (is_internalUPTEX()) wch = UCStoUTF8(wch);
                         imb = 0;  memset(mbstr, '\0', 4);
@@ -1921,6 +1930,7 @@ uint work(FILE *dvi)
                         fputs2(mbstr, fp_out);
                         fprintf(fp_out, "\"\n");
                         continue;
+                      }
                     }
 #else
                     if(f_sjis){
