@@ -2,7 +2,7 @@
 ** EmSpecialHandler.cpp                                                 **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -24,16 +24,12 @@
 #include "InputReader.hpp"
 #include "Length.hpp"
 #include "SpecialActions.hpp"
-#include "XMLNode.hpp"
-#include "XMLString.hpp"
+#include "SVGElement.hpp"
+#include "SVGTree.hpp"
 
 using namespace std;
 
 
-EmSpecialHandler::EmSpecialHandler () : _linewidth(0.4*72/72.27) {
-}
-
-
 /** Computes the "cut vector" that is used to compute the line shape.
  *  Because each line has a width > 0 the actual shape of the line is a tetragon.
  *  The 4 vertices can be influenced by the cut parameter c that specifies
@@ -78,17 +74,21 @@ static DPair cut_vector (char cuttype, const DPair &linedir, double linewidth) {
  * @param[in] lw line width in PS point units
  * @param[in] actions object providing the actions that can be performed by the SpecialHandler */
 static void create_line (const DPair &p1, const DPair &p2, char c1, char c2, double lw, SpecialActions &actions) {
-	unique_ptr<XMLElementNode> node;
+	if (actions.outputLocked())
+		return;
+	unique_ptr<SVGElement> node;
 	DPair dir = p2-p1;
 	if (dir.x() == 0 || dir.y() == 0 || (c1 == 'p' && c2 == 'p')) {
 		// draw regular line
-		node = util::make_unique<XMLElementNode>("line");
+		node = util::make_unique<SVGElement>("line");
 		node->addAttribute("x1", p1.x());
 		node->addAttribute("y1", p1.y());
 		node->addAttribute("x2", p2.x());
 		node->addAttribute("y2", p2.y());
-		node->addAttribute("stroke-width", lw);
-		node->addAttribute("stroke", actions.getColor().svgColorString());
+		node->setStrokeWidth(lw);
+		node->setStrokeColor(actions.getColor());
+		node->setStrokeOpacity(actions.getOpacity());
+
 		// update bounding box
 		DPair cv = cut_vector('p', dir, lw);
 		actions.embed(p1+cv);
@@ -98,26 +98,26 @@ static void create_line (const DPair &p1, const DPair &p2, char c1, char c2, dou
 	}
 	else {
 		// draw polygon
+		vector<DPair> points;
 		DPair cv1 = cut_vector(c1, dir, lw);
 		DPair cv2 = cut_vector(c2, dir, lw);
-		DPair q11 = p1+cv1, q12 = p1-cv1;
-		DPair q21 = p2+cv2, q22 = p2-cv2;
-		ostringstream oss;
-		oss << XMLString(q11.x()) << ',' << XMLString(q11.y()) << ' '
-			 << XMLString(q12.x()) << ',' << XMLString(q12.y()) << ' '
-			 << XMLString(q22.x()) << ',' << XMLString(q22.y()) << ' '
-			 << XMLString(q21.x()) << ',' << XMLString(q21.y());
-		node = util::make_unique<XMLElementNode>("polygon");
-		node->addAttribute("points", oss.str());
-		if (actions.getColor() != Color::BLACK)
-			node->addAttribute("fill", actions.getColor().svgColorString());
+		points.push_back(p1+cv1);
+		points.push_back(p1-cv1);
+		points.push_back(p2-cv2);
+		points.push_back(p2+cv2);
+
+		node = util::make_unique<SVGElement>("polygon");
+		node->setPoints(points);
+		node->setFillColor(actions.getColor());
+		node->setFillOpacity(actions.getOpacity());
+
 		// update bounding box
-		actions.embed(q11);
-		actions.embed(q12);
-		actions.embed(q21);
-		actions.embed(q22);
+		actions.embed(points[0]);
+		actions.embed(points[1]);
+		actions.embed(points[2]);
+		actions.embed(points[3]);
 	}
-	actions.appendToPage(std::move(node));
+	actions.svgTree().appendToPage(std::move(node));
 }
 
 
@@ -126,8 +126,8 @@ static void create_line (const DPair &p1, const DPair &p2, char c1, char c2, dou
 static double read_length (InputReader &in) {
 	double val = in.getDouble();
 	string unitstr;
-	if (isalpha(in.peek())) unitstr += in.get();
-	if (isalpha(in.peek())) unitstr += in.get();
+	if (isalpha(in.peek())) unitstr += char(in.get());
+	if (isalpha(in.peek())) unitstr += char(in.get());
 	Length::Unit unit = Length::Unit::PT;
 	try {
 		unit = Length::stringToUnit(unitstr);
@@ -163,7 +163,7 @@ bool EmSpecialHandler::process (const string &prefix, istream &is, SpecialAction
 		{"moveto",    &EmSpecialHandler::moveto},
 		{"lineto",    &EmSpecialHandler::lineto},
 		{"linewidth", &EmSpecialHandler::linewidth},
-		{0, 0}
+		{nullptr, nullptr}
 	};
 
 	StreamInputReader ir(is);
@@ -237,7 +237,7 @@ void EmSpecialHandler::line (InputReader &ir, SpecialActions& actions) {
 		// Line endpoints don't necessarily have to be defined before
 		// a line definition. If a point isn't defined yet, we put the line
 		// in a wait list and process the lines at the end of the page.
-		_lines.emplace_back(Line(pointnum1, pointnum2, char(cut1), char(cut2), linewidth));
+		_lines.emplace_back(pointnum1, pointnum2, char(cut1), char(cut2), linewidth);
 	}
 }
 
