@@ -2,7 +2,7 @@
 ** FileSystem.cpp                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -20,20 +20,20 @@
 
 #include <config.h>
 #include <algorithm>
+#include <chrono>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
 #include "FileSystem.hpp"
 #include "utility.hpp"
 #include "version.hpp"
+#include "XXHashFunction.hpp"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
 
-using namespace std;
-
 #ifdef _WIN32
 	#include <direct.h>
 	#include "windows.hpp"
@@ -50,8 +50,10 @@ using namespace std;
 #endif
 
 
+using namespace std;
+
 string FileSystem::TMPDIR;
-const char *FileSystem::TMPSUBDIR = nullptr;
+FileSystem::TemporaryDirectory FileSystem::_tmpdir;
 
 
 /** Private wrapper function for mkdir: creates a single folder.
@@ -79,13 +81,6 @@ static bool inline s_rmdir (const string &dirname) {
 }
 
 
-FileSystem::~FileSystem () {
-	// remove the subdirectory from the system's temp folder (if empty)
-	if (TMPSUBDIR)
-		s_rmdir(tmpdir());
-}
-
-
 bool FileSystem::remove (const string &fname) {
 	return unlink(fname.c_str()) == 0;
 }
@@ -140,16 +135,38 @@ string FileSystem::ensureForwardSlashes (string path) {
 }
 
 
+/** Returns the absolute path of the current working directory. */
 string FileSystem::getcwd () {
 	char buf[1024];
 #ifdef _WIN32
-	return ensureForwardSlashes(_getcwd(buf, 1024));
+	GetCurrentDirectoryA(1024, buf);
+	return ensureForwardSlashes(buf);
 #else
 	return ::getcwd(buf, 1024);
 #endif
 }
 
 
+#ifdef _WIN32
+/** Returns the absolute path of the current directory of a given drive.
+ *  Windows keeps a current directory for every drive, i.e. when accessing a drive
+ *  without specifying a path (e.g. with "cd z:"), the current directory of that
+ *  drive is used.
+ *  @param[in] drive letter of drive to get the current directory from
+ *  @return absolute path of the directory */
+string FileSystem::getcwd (char drive) {
+	string cwd = getcwd();
+	if (cwd.length() > 1 && cwd[1] == ':' && tolower(cwd[0]) != tolower(drive)) {
+		chdir(string(1, drive)+":");
+		string cwd2 = cwd;
+		cwd = getcwd();
+		chdir(string(1, cwd2[0])+":");
+	}
+	return cwd;
+}
+#endif
+
+
 /** Changes the work directory.
  *  @param[in] dir path to new work directory
  *  @return true on success */
@@ -192,36 +209,29 @@ const char* FileSystem::userdir () {
 
 /** Returns the path of the temporary folder. */
 string FileSystem::tmpdir () {
-	string ret;
-	if (!TMPDIR.empty())
-		ret = TMPDIR;
-	else {
+	if (_tmpdir.path().empty()) {
+		string basedir;
+		if (!TMPDIR.empty())
+			basedir = TMPDIR;
+		else {
 #ifdef _WIN32
-		char buf[MAX_PATH];
-		if (GetTempPath(MAX_PATH, buf))
-			ret = ensureForwardSlashes(buf);
-		else
-			ret = ".";
+			char buf[MAX_PATH];
+			if (GetTempPath(MAX_PATH, buf))
+				basedir = ensureForwardSlashes(buf);
+			else
+				basedir = ".";
 #else
-		if (const char *path = getenv("TMPDIR"))
-			ret = path;
-		else
-			ret = "/tmp";
+			if (const char *path = getenv("TMPDIR"))
+				basedir = path;
+			else
+				basedir = "/tmp";
 #endif
-		if (ret.back() == '/')
-			ret.pop_back();
-		static bool initialized=false;
-		if (!initialized && ret != ".") {
-			TMPSUBDIR = PROGRAM_NAME;
-			s_mkdir(ret + "/" + TMPSUBDIR);
-			initialized = true;
+			if (basedir.back() == '/')
+				basedir.pop_back();
 		}
-		if (TMPSUBDIR)
-			ret += string("/") + TMPSUBDIR;
+		_tmpdir = TemporaryDirectory(basedir, PROGRAM_NAME);
 	}
-	if (!ret.empty() && ret.back() != '/')
-		ret += '/';
-	return ret;
+	return _tmpdir.path();
 }
 
 
@@ -368,3 +378,33 @@ int FileSystem::collect (const std::string &dirname, vector<string> &entries) {
 #endif
 	return entries.size();
 }
+
+
+/** Creates a temporary directory in a given folder.
+ *  @param[in] folder folder path in which the directory is to be created
+ *  @param[in] prefix initial string of the directory name */
+FileSystem::TemporaryDirectory::TemporaryDirectory (const std::string &folder, string prefix) {
+	using namespace std::chrono;
+	auto now = system_clock::now().time_since_epoch();
+	auto now_ms = duration_cast<milliseconds>(now).count();
+	auto hash = XXH64HashFunction(to_string(now_ms)).digestValue();
+	if (!prefix.empty() && prefix.back() != '-')
+		prefix += "-";
+	for (int i=0; i < 10 && _path.empty(); i++) {
+		hash++;
+		stringstream oss;
+		oss << folder << '/' << prefix << hex << hash;
+		if (exists(oss.str()))
+			continue;
+		if (s_mkdir(oss.str()))
+			_path = oss.str() + "/";
+		else
+			break;
+	}
+}
+
+
+FileSystem::TemporaryDirectory::~TemporaryDirectory () {
+	if (!_path.empty())
+		s_rmdir(_path);
+}
