@@ -2,7 +2,7 @@
 ** Ghostscript.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -66,7 +66,7 @@ static string get_path_from_registry () {
 					if (major_version >= 7) {
 						char dll_path[512];  // path to Ghostscript DLL stored in the registry
 						DWORD length = 512;
-						if (RegGetValueA(hkey, subkey, "GS_DLL", RRF_RT_REG_SZ, 0, dll_path, &length) == ERROR_SUCCESS) {
+						if (RegGetValueA(hkey, subkey, "GS_DLL", RRF_RT_REG_SZ, nullptr, dll_path, &length) == ERROR_SUCCESS) {
 							RegCloseKey(hkey);
 							return dll_path;
 						}
@@ -144,6 +144,9 @@ static string get_libgs (const string &fname) {
 		dlname = "libgs." + to_string(i) + ".dylib";
 		if (loader.loadLibrary(dlname))
 			return dlname;
+		dlname = "libgs.dylib." + to_string(i);
+		if (loader.loadLibrary(dlname))
+			return dlname;
 #endif
 	}
 #endif
@@ -160,7 +163,6 @@ Ghostscript::Ghostscript ()
 	: DLLoader(get_libgs(LIBGS_NAME))
 #endif
 {
-	_inst = 0;
 }
 
 
@@ -173,7 +175,6 @@ Ghostscript::Ghostscript (int argc, const char **argv, void *caller)
 	: DLLoader(get_libgs(LIBGS_NAME))
 #endif
 {
-	_inst = 0;
 	init(argc, argv, caller);
 }
 
@@ -191,12 +192,12 @@ bool Ghostscript::init (int argc, const char **argv, void *caller) {
 	if (!_inst) {
 		int status = new_instance(&_inst, caller);
 		if (status < 0)
-			_inst = 0;
+			_inst = nullptr;
 		else {
 			init_with_args(argc, const_cast<char**>(argv));
 		}
 	}
-	return _inst != 0;
+	return _inst != nullptr;
 }
 
 
@@ -214,7 +215,7 @@ bool Ghostscript::available () {
 /** Retrieves version information about Ghostscript.
  * @param[out] r takes the revision information (see GS API documentation for further details)
  * @return true on success  */
-bool Ghostscript::revision (gsapi_revision_t *r) {
+bool Ghostscript::revision (gsapi_revision_t *r) const {
 #if defined(HAVE_LIBGS)
 	return (gsapi_revision(r, sizeof(gsapi_revision_t)) == 0);
 #else
@@ -226,7 +227,7 @@ bool Ghostscript::revision (gsapi_revision_t *r) {
 
 
 /** Returns the revision number of the GS library. */
-int Ghostscript::revision () {
+int Ghostscript::revision () const {
 	gsapi_revision_t r;
 	if (revision(&r))
 		return static_cast<int>(r.revision);
@@ -238,10 +239,18 @@ int Ghostscript::revision () {
 string Ghostscript::revisionstr () {
 	string revstr;
 	if (int rev = revision()) {
-		revstr = to_string(rev/100) + ".";
-		if (rev % 100 < 10)
-			revstr += "0";
-	  	revstr += to_string(rev%100);
+		if (rev < 1000) {  // until GS 9.52
+			revstr = to_string(rev/100) + ".";
+			if (rev % 100 < 10)
+				revstr += "0";
+			revstr += to_string(rev%100);
+		}
+		else { // as of GS 9.52.1, see ghostpdl/base/gsmisc.c
+			int major = rev / 1000;
+			int minor = (rev - major*1000)/10;
+			int patch = rev % 10;
+			revstr = to_string(major) + "." + to_string(minor) + "." + to_string(patch);
+		}
 	}
 	return revstr;
 }
@@ -257,7 +266,7 @@ int Ghostscript::new_instance (void **psinst, void *caller) {
 #else
 	if (auto fn = LOAD_SYMBOL(gsapi_new_instance))
 		return fn(psinst, caller);
-	*psinst = 0;
+	*psinst = nullptr;
 	return 0;
 #endif
 }
