@@ -2,7 +2,7 @@
 ** SVGOutput.cpp                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -139,6 +139,11 @@ string SVGOutput::expandFormatString (string str, int page, int numPages, const
 					}
 					break;
 				}
+				case 'n':
+				case 'N':
+					oss << (str[pos] == 'n' ? _fileNumber : _fileCount);
+					result += oss.str();
+					break;
 				case 'p':
 				case 'P':
 					oss << (str[pos] == 'p' ? page : numPages);
