@@ -2,7 +2,7 @@
  * Gregorio is a program that translates gabc files to GregorioTeX
  * This header defines the Gregorio data structures and functions.
  *
- * Copyright (C) 2006-2017 The Gregorio Project (see CONTRIBUTORS.md)
+ * Copyright (C) 2006-2021 The Gregorio Project (see CONTRIBUTORS.md)
  *
  * This file is part of Gregorio.
  *
@@ -102,6 +102,7 @@ ENUM(gregorio_clef, GREGORIO_CLEF);
     E(S_PUNCTUM) \
     E(S_PUNCTUM_END_OF_GLYPH) \
     E(S_PUNCTUM_INCLINATUM_ASCENDENS) \
+    E(S_PUNCTUM_INCLINATUM_STANS) \
     E(S_PUNCTUM_INCLINATUM_DESCENDENS) \
     E(S_PUNCTUM_INCLINATUM_DEMINUTUS) \
     E(S_PUNCTUM_INCLINATUM_AUCTUS) \
@@ -121,8 +122,11 @@ ENUM(gregorio_clef, GREGORIO_CLEF);
     E(S_TRISTROPHA) \
     E(S_LINEA_PUNCTUM) \
     E(S_FLAT) \
+    E(S_FLAT_PAREN) \
     E(S_SHARP) \
+    E(S_SHARP_PAREN) \
     E(S_NATURAL) \
+    E(S_NATURAL_PAREN) \
     /* special shapes that must not appear in the final form of the score :
      * quadratum is the shape of the first note of a punctum quadratum
      * and quilisma quadratum is the shape of the first note of a pes
@@ -155,7 +159,16 @@ ENUM(gregorio_shape, GREGORIO_SHAPE);
     E(B_DIVISIO_MINOR_D5) \
     E(B_DIVISIO_MINOR_D6) \
     E(B_DIVISIO_MINOR_D7) \
-    L(B_DIVISIO_MINOR_D8)
+    E(B_DIVISIO_MINOR_D8) \
+    E(B_VIRGULA_HIGH) \
+    E(B_DIVISIO_MINIMA_HIGH) \
+    E(B_DIVISIO_MAIOR_DOTTED) \
+    E(B_DIVISIO_MINIMIS) \
+    E(B_DIVISIO_MINIMIS_HIGH) \
+    E(B_VIRGULA_PAREN) \
+    E(B_VIRGULA_PAREN_HIGH) \
+    E(B_DIVISIO_MINIMA_PAREN) \
+    L(B_DIVISIO_MINIMA_PAREN_HIGH)
 ENUM(gregorio_bar, GREGORIO_BAR);
 
 /* definition of the signs. You can notice that the values are made so
@@ -395,7 +408,7 @@ ENUM(gregorio_ledger_specificity, GREGORIO_LEDGER_SPECIFICITY);
 
 typedef struct gregorio_extra_info {
     char *ad_hoc_space_factor;
-    ENUM_BITFIELD(gregorio_bar) bar:4;
+    ENUM_BITFIELD(gregorio_bar) bar:5;
     ENUM_BITFIELD(gregorio_space) space:4;
     ENUM_BITFIELD(gregorio_nlba) nlba:2;
     bool eol_ragged:1;
