@@ -3,7 +3,7 @@
 # latexdiff-vc  - wrapper script for applying latexdiff to rcs managed files
 #                 and for automatised creation of postscript or pdf from difference file
 #
-#   Copyright (C) 2005-16  F J Tilmann (tilmann@gfz-potsdam.de)
+#   Copyright (C) 2005-20  F J Tilmann (tilmann@gfz-potsdam.de)
 #
 # Repository:         https://github.com/ftilmann/latexdiff
 # CTAN page:          http://www.ctan.org/pkg/latexdiff
@@ -26,18 +26,24 @@
 # Detailed usage information at the end of the file
 #
 # TODO/IDEAS: - option to call external pre-processing codes
-#             - choose type of latex processor / bibtex (luatex, xelatex etc)
+# version 1.3.2:
+#    - bug fix: when setting config variables with the command lines it is now possible to use quotes to includes spaces in the value, e.g. "-c LATEX=pdflatex --shell-escape'
+#    - bug fix: --only-changes is not compatible with graphics markup. --graphics-markup=none is now set automatically
+#    - when --pdf or --postscript is select, then replace tildes in version names with underscores in output file (e.g. diffHEAD_1 instead of diff HEAD~1. This is necessary because pdflatex strips ~n strings at the end of the filename, when generating output file names
+
+# version 1.3.1:
+#    - bug fix, import File::Path explicitly as otherwise mkpath is not found
 # version 1.3.0 ((7 October 2018)
 #    - option --only-changes with hyperref will suppress hyperrefs (pull request jprotze)_
 #    - option --only-changes now moves (rather than copies) file with only changes 
 # version 1.2.1 (22 June 2017):
-#    - update: use qpdf (instead of pdftk) to select particular pages of output (pull request #102 submited by Tom Scogland via github). This was necessary because pdftk is being deprecated. pdftk is still used as a fall-back
+#    - update: use qpdf (instead of pdftk) to select particular pages of output (pull request #102 submitted by Tom Scogland via github). This was necessary because pdftk is being deprecated. pdftk is still used as a fall-back
 #    - bug fix: --hg option was not recognised (partially fixes github issue #93 )
 #    - wrap passed-through options to latexdiff in quotation marks (fix github issue #58 )
 #    - program names for latexdiff, latex, dvips, bibtex configurable (fixes issue #40)
 #
 # version 1.2.0:
-#    - depracation fix: left brace in RegEx now needs to be escaped
+#    - deprecation fix: left brace in RegEx now needs to be escaped
 #
 # version 1.1.1:
 #    - better detection of RCS system 
@@ -67,13 +73,14 @@ use Pod::Usage qw/pod2usage/ ;
 use File::Temp qw/tempdir/ ;
 use File::Basename qw/dirname/;
 use File::Copy;
+use File::Path;
 
 use strict ;
 use warnings ;
 
 my $versionstring=<<EOF ;
-This is LATEXDIFF-VC 1.3.0
-  (c) 2005-2018 F J Tilmann
+This is LATEXDIFF-VC 1.3.2
+  (c) 2005-2021 F J Tilmann
 EOF
 
 # output debug and intermediate files, set to 0 in final distribution
@@ -206,7 +213,8 @@ foreach $config ( @configlist ) {
 
 $configlatexdiff="";
 foreach $assign ( @config ) {
-  $assign=~ m/\s*(\w*)\s*=\s*(\S*)\s*$/ or die "Illegal assignment $assign in configuration list (must be variable=value)";  
+    $assign=~ m/\s*(\w*)\s*=\s*(.*?)\s*$/ 
+      or die "Illegal assignment |$assign| in configuration list (must be variable=value)";  
   if ( defined($CFG{$1})) {
     # known latexdiff-vc option
     $CFG{$1}=$2;
@@ -316,11 +324,11 @@ if ( scalar(@revs)>0 ) {
 
 # make file list (last arguments), initial arguments have to be passed to latexdiff
 # We assume an argument is a valid file rather than a latexdiff argument
-# if it has extension .tex, .bbl or .flt (output of flatex)
+# if it has extension .tex, .bbl or .flt (output of flatex), and does not start with a - or --
 
 @files=($file2);
 while( $file1=pop @ARGV ) {
-  if ( $file1 =~ /\.(tex|bbl|flt)$/ ) {
+  if ( $file1 =~ /\.(tex|bbl|flt)$/ && $file1 !~ /^-/) {
     # $file1 looks like a valid file name and is prepended to file list
     unshift @files, $file1 ;
   } else {
@@ -343,7 +351,7 @@ if ( defined($debug) && $debug ) {
 
 # impose ZLABEL subtype if --only-changes option
 if ( $onlychanges ) {
-  push @ldoptions, "-s", "ZLABEL","-f","IDENTICAL","--no-links" ;
+  push @ldoptions, "-s", "ZLABEL","-f","IDENTICAL","--no-links", "--graphics-markup=none" ;
 }
 
 if ( scalar(@revs) == 0 ) {
@@ -379,6 +387,12 @@ if ( scalar(@revs) == 2 ) {
   $append = "-diff";
 }
 
+if ( defined($pdf) or defined($postscript) ) {
+  # when using options such as -r HEAD~1  the ~1 is ignored by pdflatex in generating output file names; this could
+  # be confusing and actually causes error to only-changes mode
+  $append =~ s/~/_/g ;
+}
+
 if ( defined ($dir) && ! $dir ) {
   # bare -d option => choose directory name
   ($dir=$append) =~ s/^-//;
@@ -533,7 +547,7 @@ foreach $diff ( @difffiles ) {
       #print STDERR "Generated postscript file $ps\n";
     } elsif ( $run ) {
       if ( $onlychanges ) {
-	my @pages=findchangedpages("$diffbase.aux");
+	my @pages=compresspages(findchangedpages("$diffbase.aux"));
         my $gs = `which gs`;
         $gs =~ s/^\s+|\s+$//g;
         my $qpdf = `which qpdf`;
@@ -541,7 +555,7 @@ foreach $diff ( @difffiles ) {
         my $pdftk = `which pdftk`;
         $pdftk =~ s/^\s+|\s+$//g;
         my $command;
-        if (-x $gs && `gs --version` >= 9.20) {
+        if (-x $gs && `$gs --version` >= 9.20) {
           $command="gs -sDEVICE=pdfwrite -dNOPAUSE -dBATCH -dSAFER -sPageList=" . join(",", @pages) . " -sOutputFile=\"$diffbase-changedpage.pdf\" \"$diffbase.pdf\"";
         } elsif (-x $pdftk) {
           $command="pdftk \"$diffbase.pdf\" cat " . join(" ",@pages) . " output \"$diffbase-changedpage.pdf\"";
@@ -589,13 +603,52 @@ sub findchangedpages {
   return(sort {$a <=> $b} keys(%pages));
 }
 
+# inspired by a python version posted at https://stackoverflow.com/a/54714846/8136338
+# compresspages(@pages)
+# @pages: sorted array of unique integers (pages)
+# return a list of pages and page ranges. 3 or more consecutive numbers are merged into a "begin-end" string
+#
+# example: join(",", compresspages([1,3,4,5,7,8])) -> join(",", [1,"3-5",7,8]) -> "1,3-5,7,8"
+sub compresspages {
+  my (@pages) = @_;
+  my @res;
+  my $begin=$pages[0];
+  my $end =$pages[0];
+  my $page;
+  foreach $page ( @pages ) {
+    next if ($page == $begin);
+    if ($page == $end+1) { # handle continuous pages
+      $end = $page;
+      next;
+    } elsif ($begin == $end) { # push single continuous page
+      push @res, $begin;
+      $begin = $end = $page;
+    } elsif ($begin + 1 == $end) { # push two continuous pages
+      push @res, $begin, $end;
+      $begin = $end = $page;
+    } else { # push multiple continuous pages
+      push @res, "$begin-$end";
+      $begin = $end = $page;
+    }
+  }
+  # Finally just check in the same manner for the end element
+  if ($begin == $end) {
+    push @res, $begin;
+  } elsif ($begin + 1 == $end) {
+    push @res, $begin, $end;
+  } else {
+    push @res, "$begin-$end";
+  }
+  return @res;
+}
+
 # checkout_dir(rev,dirname)
 # checks out revision rev and stores it in dirname
 # uses global variables: $vc, $rootdir
 sub checkout_dir {
   my ($rev,$dirname)=@_;
 
-  unless (-e $dirname) { mkdir $dirname or die "Cannot mkdir $dirname ." ;}
+  unless (-e $dirname) { mkpath([ $dirname ]) or die "Cannot mkdir $dirname ." ;}
   if ( $vc eq "SVN" ) {
     system("svn checkout -r $rev $rootdir $dirname")==0 or die "Something went wrong in executing:  svn checkout -r $rev $rootdir $dirname";
   } elsif ( $vc eq "GIT" ) {
@@ -674,7 +727,7 @@ specified, and they result in different behaviour:
 
 =item B<latexdiff-vc> -r F<file.tex> ...
 
-compares F<file.tex> with the most recent version checked into RCS.
+compares F<file.tex> with the most recently checked-in version checked.
 
 =item B<latexdiff-vc> -r F<rev1> F<file.tex> ...
 
@@ -685,7 +738,7 @@ compares F<file.tex> with revision F<rev1>.
 compares revisions F<rev1> and F<rev2> of F<file.tex>.
 
 Multiple files can be specified for all of the above options. All files must have the
-extension C<.tex>, though.
+extensions C<.tex>, C<.bbl>, or C<.flt>, though.
 
 =item B<latexdiff-vc>  F<old.tex> F<new.tex>
 
@@ -701,7 +754,7 @@ reported to stdout.
 Rather than appending the string C<diff> and optionally the version
 numbers given to the output-file, this will prepend a directory name C<diff> 
 to the
-original filename, creating the directory and subdirectories should they not exist already.  This is particularly useful in order to clone a
+original filename, creating the directory and sub-directories should they not exist already.  This is particularly useful in order to clone a
 complete directory hierarchy.  Optionally, a pathname F<path> can be specified, which is prepended instead of C<diff>.
 
 =item B<--flatten,--flatten=keep-intermediate>
@@ -713,15 +766,13 @@ The generic usage of this function is : C<latexdiff-vc --flatten -r rev1 [-r rev
 
 With C<--flatten=keep-intermediate>, the intermediate revision snapshots are kept in the current directory (Default is to store them in a temporary directory and delete them after generating the diff file.)
 
-=item B<--config var1=val1,var2=val2,...> or B<-c var1=val1,..>
-
-
 
 =item B<--only-changes>
 
 Post-process the output such that only pages with changes on them are displayed. This requires the use of subtype ZLABEL 
 in latexdiff, which will be set automatically, but any manually set -s option will be overruled (also requires zref package to 
-be installed). (note that this option must be combined with --ps or --pdf to make sense)
+be installed). This option also disables internal links (as implemented by hyperref package) and graphics markup.
+(note that this option must be combined with --ps or --pdf to make sense)
 
 =item B<--force>
 
@@ -738,6 +789,7 @@ run latex command on diff file after generation of diff file.
 run latex and dvixxx commands after generation of diff file.
 
 =item B<-c configfile>
+=item B<--config var1=val1,var2=val2,...> or B<-c var1=val1,..>
 
 Set configuration variables for latexdiff and latexdiff-vc.  The option can be repeated to set different
 variables (as an alternative to the comma-separated list).
@@ -755,7 +807,7 @@ Available variables for latexdiff-vc:
 
 =back
 
-All other config variables are passed to latexdiff. Explicity set configuration changes always override implicit
+All other config variables are passed to latexdiff. Explicitly set configuration changes always override implicit
 changes by the following shortcut options B<--fast>, B<--so>, B<--ps> and B<--pdf>. 
 
 =item B<--fast> or B<--so>
@@ -803,7 +855,7 @@ L<latexdiff>
 =head1 PORTABILITY
 
 I<latexdiff-vc> uses external commands and is therefore dependent on the system architecture; it has been
-tested mainly on Unix-like systems. It also requires the a version control
+tested mainly on Unix-like systems. It also requires a version control
 system and latex to be installed on the system to make use of all features.  Modules from Perl 5.8
 or higher are required.
 
