@@ -4,9 +4,6 @@ extern UChar AIUEO[];
 extern UChar *aiueo;
 extern UChar kanatable[];
 extern UChar extkanatable[];
-extern UChar circkanatable[];
-
-extern UChar GANADA[];
 
 #define SPACE    0x3000    /* 全角スペース */
 #define ALPHATOP 0xff01    /* ！ */
@@ -20,8 +17,19 @@ extern UChar GANADA[];
 #define EXKANAEND  0x31FF  /* ㇿ */
 #define CRKANATOP  0x32D0  /* ㋐ */
 #define CRKANAEND  0x32FE  /* ㋾ */
+#define HANKANATOP 0xFF71  /* ｱ */
+#define HANKANAEND 0xFF9D  /* ﾝ */
+#define HANKANAWO  0xFF66  /* ｦ */
+#define HANKANATU  0xFF6F  /* ｯ */
+#define SQKANATOP  0x3300  /* ㌀ */
+#define SQKANAEND  0x3357  /* ㍗ */
+#define CRLATNTOP  0x24B6  /* Ⓐ */
+#define CRLATNEND  0x24E9  /* ⓩ */
 
 #define is_katakana(a)  ((a)>=KATATOP && (a)<=KATAEND)
 #define is_hiragana(a)  ((a)>=HIRATOP && (a)<=HIRAEND)
 #define is_extkana(a)   ((a)>=EXKANATOP && (a)<=EXKANAEND)
 #define is_circkana(a)  ((a)>=CRKANATOP && (a)<=CRKANAEND)
+#define is_hankana(a)   ((a)>=HANKANATOP && (a)<=HANKANAEND || (a)>=HANKANAWO && (a)<=HANKANATU)
+#define is_sqkana(a)    ((a)>=SQKANATOP && (a)<=SQKANAEND)
+#define is_circlatin(a) ((a)>=CRLATNTOP && (a)<=CRLATNEND)
