@@ -674,10 +674,10 @@ static void do_exception(halfword wordstart, halfword r, char *replacement)
             /*tex Let's check if we have a penalty spec. */
             if (((i+3) < len) && uword[i+1] == '[' && uword[i+2] >= '0' && uword[i+2] <= '9' && uword[i+3] == ']') {
                 if (exception_penalty_par > 0) {
-                    if (exception_penalty_par > 100000) {
-                        pen = (uword[i+2] - '0') * exception_penalty_par ;
-                    } else {
+                    if (exception_penalty_par > 10000) {
                         pen = exception_penalty_par;
+                    } else {
+                        pen = (uword[i+2] - '0') * exception_penalty_par ;
                     }
                 } else {
                     pen = hyphen_penalty_par;
@@ -983,7 +983,18 @@ void hnj_hyphenation(halfword head, halfword tail)
         halfword hyf_font;
         halfword end_word = r;
         wordstart = r;
-        assert(is_simple_character(wordstart));
+        /*assert(is_simple_character(wordstart));*/
+        if (!(is_simple_character(wordstart))){
+         if (!(is_character(wordstart))) {
+          tex_error("the word doesn't start with a character", NULL);
+         } else if (is_ligature(wordstart)) {
+           tex_error("the word starts with a ligature", NULL);
+         } else if (is_ghost(wordstart)) {
+           tex_error("the word starts with a ghost glyph", NULL);
+         } else {
+           tex_error("the word doesn't start with a simple character", NULL);
+         }
+        }
         hyf_font = font(wordstart);
         if (hyphen_char(hyf_font) < 0) {
             /*tex For backward compatibility we set: */
@@ -1012,6 +1023,12 @@ void hnj_hyphenation(halfword head, halfword tail)
                 break;
             }
             wordlen++;
+            if (wordlen >= MAX_WORD_LEN) {
+             while (r && type(r) == glyph_node) {
+               r = vlink(r);
+             }
+             goto PICKUP;
+            }
             if (lchar <= 32) {
                 if (lchar == 32) {
                     lchar = 0 ;
@@ -1038,7 +1055,7 @@ void hnj_hyphenation(halfword head, halfword tail)
         if (explicit_hyphen == true) {
             /*tex we are not at the start, so we only need to look ahead */
             halfword t = vlink(r) ;
-            if ((automatic_hyphen_mode_par == 0 || automatic_hyphen_mode_par == 1) && (t != null) && ((type(t) == glyph_node) && (character(t) != ex_hyphen_char_par))) {
+            if ((automatic_hyphen_mode_par == 0 || automatic_hyphen_mode_par == 1) && (t != null) && ((type(t) == glyph_node) && is_simple_character(t) && (character(t) != ex_hyphen_char_par))) {
                 /*tex we have a word already but the next character may not be a hyphen too */
                 r = compound_word_break(r, char_lang(r));
                 if (compound_hyphen) {
@@ -1052,7 +1069,7 @@ void hnj_hyphenation(halfword head, halfword tail)
                 }
             } else {
                 /*tex we jump over the sequence of hyphens */
-               while ((t != null) && (type(t) == glyph_node) && (character(t) == ex_hyphen_char_par)) {
+               while ((t != null) && (type(t) == glyph_node) && is_simple_character(t) && (character(t) == ex_hyphen_char_par)) {
                     r = t ;
                     t = vlink(r) ;
                 }
@@ -1129,6 +1146,7 @@ void hnj_hyphenation(halfword head, halfword tail)
                 }
             }
         }
+PICKUP:
         expstart = null ;
         explicit_hyphen = false;
         wordlen = 0;
