@@ -290,7 +290,8 @@ static int Isspace (char c)
 __attribute__ ((noreturn))
 static void luatex_calledit (int baseptr, int linenumber)
 {
-    char *temp, *command, *fullcmd;
+    char *temp, *command;
+    char *fullcmd = NULL; /* avoid compiler warning */
     char c;
     int sdone, ddone, i;
     char *filename = makecstring(input_stack[base_ptr].name_field);
@@ -398,8 +399,9 @@ static void luatex_calledit (int baseptr, int linenumber)
         strcat(fullcmd, "\"");
         strcat(fullcmd, command);
     }
-#endif
+#else
     fullcmd = command;
+#endif
     /*tex Execute the command. */
     if (system (fullcmd) != 0) {
         fprintf (stderr, "! Trouble executing `%s'.\n", command);
@@ -441,6 +443,21 @@ void error(void)
         /*tex Get user's advice and |return|. */
         while (1) {
           CONTINUE:
+            /*tex
+             Original reports:
+
+             https://tex.stackexchange.com/questions/551313/
+             https://tug.org/pipermail/tex-live/2020-June/045876.html
+
+            This will probably be fixed by DEK in the 2021 tuneup in a different
+            way (so we'll have to remove or alter this change), but the interaction
+            sequence in the reports above causes a segmentation fault in web2c -
+            writing to the closed \write15 stream because we wrongly decrement
+            selector from 16 to 15 in term_input, due to the lack of this check in
+            recursive error() call.
+            */
+            if (interaction !=error_stop_mode)
+                return;
             clear_for_error_prompt();
             prompt_input("? ");
             if (last == first)
@@ -879,6 +896,8 @@ When \TeX\ wants to typeset a character that doesn't exist, the character node i
 not created; thus the output routine can assume that characters exist when it
 sees them. The following procedure prints a warning message unless the user has
 suppressed it.
+If |tracing_lost_chars_par| (i.e. \.{\\tracinglostchar})  is  greater than 2,
+it's considered as an error.
 
 */
 
@@ -915,6 +934,9 @@ void char_warning(internal_font_number f, int c)
         end_diagnostic(false);
         tracing_online_par = old_setting;
     }
+    if (tracing_lost_chars_par > 2) {
+       error();
+    }
 }
 
 void wrapup_backend(void) {
@@ -965,7 +987,7 @@ void normal_warning(const char *t, const char *p)
         new_line_char_par = 10;
         report_id = callback_defined(show_lua_error_hook_callback);
         if (report_id == 0) {
-	    if (p != NULL) 
+	    if (p != NULL)
 	      tprint(p);
             help2(
                 "The lua interpreter ran into a problem, so the",
