@@ -382,7 +382,8 @@ void print_nlp(void)
 {
     if (new_string_line > 0) {
         print_char(new_string_line);
-    } else if (((term_offset > 0) && (odd(selector))) ||
+    } else if ((selector < no_print) ||
+               ((term_offset > 0) && (odd(selector))) ||
                ((file_offset > 0) && (selector >= log_only))) {
         print_ln();
     }
@@ -466,6 +467,7 @@ void tprint(const char *sss)
     /*tex What is left is the 3 term/log settings. */
     if (dolog || doterm) {
         buffer = xmalloc(strlen(sss)*3);
+        buffer[0] = '\0';
         /*tex The |wrapup_run| callback acts when the log file is already closed.*/
         if (dolog && log_opened_global) {
             const unsigned char *ss = (const unsigned char *) sss;
@@ -1297,6 +1299,49 @@ void end_diagnostic(boolean blank_line)
     selector = global_old_setting;
 }
 
+/*
+    Indentation (based on a suggestion by PO on tex-implementors but adapted to
+    out situation and with built-in limits and so.
+*/
+
+void print_input_level(void)
+{
+    int callback_id = callback_defined(input_level_string_callback);
+    if (callback_id>0) {
+        char *s = NULL;
+        if (run_callback(callback_id, "d->S", input_ptr, &s)) {
+            if (s && strlen(s) > 0) {
+                tprint_nl(s);
+                free(s);
+            } else {
+                print_ln();
+            }
+        } else {
+            print_ln();
+        }
+    } else {
+        int m = level_max;
+        if (m) {
+            int l = input_ptr;
+            int c = level_chr > 0 ? level_chr : '.';
+            if (l > m) {
+                tprint_nl("[");
+                print_int((l/m)*m);
+                print(']');
+                l = l % m;
+            } else {
+                print_ln();
+            }
+            while (l > 0) {
+               print(c);
+               l--;
+            }
+        } else {
+            print_ln();
+        }
+    }
+}
+
 /*tex
 
 Of course we had better declare another global variable, if the previous routines
