@@ -316,7 +316,7 @@ boolean lua_a_open_in(alpha_file * f, char *fn, int n)
         }
     } else {
         /*tex no read callback */
-        if (openinnameok(fnam)) {
+        if (openinnameok(fn)) {
             ret = open_in_or_pipe(f, fnam, kpse_tex_format, FOPEN_RBIN_MODE, (n == 0 ? true : false));
         } else {
             /*tex open failed */
@@ -637,8 +637,11 @@ void term_input(void)
     int k;
     /*tex Now the user sees the prompt for sure: */
     update_terminal();
-    if (!input_ln(term_in, true))
+    if (!input_ln(term_in, true)) {
+        /*tex  initialize limit to avoid printing some garbage. */
+        ilimit=0;
         fatal_error("End of file on the terminal!");
+    }
     /*tex The user's line ended with \.{<return>}: */
     term_offset = 0;
     /*tex Prepare to echo the input. */
